public class ShapesProject {
	
	public static void main(String[] args){
		Square[] squares = new Square[3];
		squares[0] = new Square(1);
		squares[1] = new Square(2);
		squares[2] = new Square(3);
		
		Shape[] shapes = new Shape[5];
		shapes[0] = new Shape(1);		
		shapes[1] = new Shape(2);
		shapes[2] = new Shape(3);
		shapes[3] = new Square(4);
		shapes[4] = new Circle(5);
		
		System.out.println(findMax(squares));
		System.out.println(findMax(shapes));
	}
	
	// This works for Shapes, Circles and Squares and any future 
	// class that may extend Shape.
	// It works because of inheritance.
	// This method would not work for anything else.
//	public static Shape findMax(Shape[] a){
	
	// This works, but we are not using generics
//	public static Comparable<Shape> findMax(Comparable[] a){
		
	// Since Shape implements Comparable<Shape>, let's use Comparable<Shape>
	// This does not work.
	// Look at the "compareTo" method in the Shape class, 
	// it takes a parameter of type "Shape". This is what we expect
	// when we call "compareTo" below.
	// However, according to the parameter definition in the method 
	// header below, we provide it with an array that has elements of
	// type "Comparable<Shape>"
	// Clearly, "Shape" and "Comparable<Shape>" are not the same.
	// We need to figure out a way to fix this.
//	public static Comparable<Shape> findMax(Comparable<Shape>[] a){
	
	// This is just a different notation of the above.
	// Remember that we can instantiate a variable with the specified
	// type or any other type that extends the specified type.
	// So all we do below is state the obvious, i.e. we can instantiate
	// the array with anything that extends "Comparable<Shape>"
	// It is no surprise then that the error is not fixed.
	// However, the notation comes in handy.
//	public static <T extends Comparable<Shape>> T findMax(T[] a){

	// We know that "Shape" implements "Comparable<Shape>"
	// Since each type trivially extends its own self, 
	// we also know that "Shape" extends "Comparable<Shape>"
	// Now, things work, because Java picks the Shape type that
	// it needs for the "compareTo" method below.
	// The "Squares" class does not have its own "compareTo" method,
	// remember that it inherits the "compareTo" method from the 
	// "Shape" class. But that method takes "Shapes", not "Squares".
	// This means that we cannot call findMax with an array of
	// "Squares."
//	public static <T extends Comparable<T>> T findMax(T[] a){

    // Now, it works.
	// If we are passing in an array of squares, then "Square" is
	// assigned to T.
	// We are stating that some super class of "Square" needs to 
	// have a "compareTo" method of some super type of "Square".
	// "Shape" is a super class of "Square".
	// "Shape" has a compareTo method that takes a "Shape" parameter.
	// That "Shape" parameter is a super type of the "Square" parameter.
	public static <T extends Comparable<? super T>> T findMax(T[] a){
		int maxIndex = 0;
		for (int i = 1; i < a.length; i++){
			if (a[i].compareTo(a[maxIndex]) > 0) maxIndex = i;
		}
		return a[maxIndex];
	}
}
